@@ -30,13 +30,14 @@ from plugwise.constants import (
     ActuatorDataType,
     ActuatorType,
     ApplianceType,
-    GatewayData,
     GwEntityData,
     SensorType,
     ThermoLoc,
 )
 from plugwise.util import (
+    collect_power_values,
     common_match_cases,
+    count_data_items,
     format_measure,
     skip_obsolete_measurements,
     version_to_model,
@@ -63,32 +64,15 @@ class SmileLegacyHelper(SmileCommon):
     def __init__(self) -> None:
         """Set the constructor for this class."""
         self._appliances: etree
-        self._count: int
-        self._domain_objects: etree
-        self._heater_id: str
-        self._home_location: str
         self._is_thermostat: bool
-        self._last_modified: dict[str, str] = {}
         self._loc_data: dict[str, ThermoLoc]
         self._locations: etree
         self._modules: etree
-        self._notifications: dict[str, dict[str, str]] = {}
-        self._on_off_device: bool
-        self._opentherm_device: bool
-        self._outdoor_temp: float
-        self._status: etree
         self._stretch_v2: bool
-        self._system: etree
-
-        self.gateway_id: str
-        self.gw_data: GatewayData = {}
         self.gw_entities: dict[str, GwEntityData] = {}
-        self.smile_fw_version: Version | None
-        self.smile_hw_version: str | None
         self.smile_mac_address: str | None
         self.smile_model: str
-        self.smile_name: str
-        self.smile_type: str
+        self.smile_version: Version
         self.smile_zigbee_mac_address: str | None
         SmileCommon.__init__(self)
 
@@ -115,7 +99,7 @@ class SmileLegacyHelper(SmileCommon):
             ):
                 continue  # pragma: no cover
 
-            appl.location = self._home_location
+            appl.location = self._home_loc_id
             appl.entity_id = appliance.attrib["id"]
             appl.name = appliance.find("name").text
             # Extend device_class name when a Circle/Stealth is type heater_central -- Pw-Beta Issue #739
@@ -161,7 +145,7 @@ class SmileLegacyHelper(SmileCommon):
 
         # Legacy Anna without outdoor_temp and Stretches have no locations, create fake location-data
         if not (locations := self._locations.findall("./location")):
-            self._home_location = FAKE_LOC
+            self._home_loc_id = FAKE_LOC
             self._loc_data[FAKE_LOC] = {"name": "Home"}
             return
 
@@ -174,11 +158,11 @@ class SmileLegacyHelper(SmileCommon):
                 continue
 
             if loc.name == "Home":
-                self._home_location = loc.loc_id
+                self._home_loc_id = loc.loc_id
             # Replace location-name for P1 legacy, can contain privacy-related info
             if self.smile_type == "power":
                 loc.name = "Home"
-                self._home_location = loc.loc_id
+                self._home_loc_id = loc.loc_id
 
             self._loc_data[loc.loc_id] = {"name": loc.name}
 
@@ -187,15 +171,15 @@ class SmileLegacyHelper(SmileCommon):
 
         Use the home_location or FAKE_APPL as entity id.
         """
-        self.gateway_id = self._home_location
+        self.gateway_id = self._home_loc_id
         if self.smile_type == "power":
             self.gateway_id = FAKE_APPL
 
         self.gw_entities[self.gateway_id] = {"dev_class": "gateway"}
         self._count += 1
         for key, value in {
-            "firmware": str(self.smile_fw_version),
-            "location": self._home_location,
+            "firmware": str(self.smile_version),
+            "location": self._home_loc_id,
             "mac_address": self.smile_mac_address,
             "model": self.smile_model,
             "name": self.smile_name,
@@ -272,7 +256,7 @@ class SmileLegacyHelper(SmileCommon):
         Collect the appliance-data based on entity_id.
         """
         data: GwEntityData = {"binary_sensors": {}, "sensors": {}, "switches": {}}
-        # Get P1 smartmeter data from LOCATIONS or MODULES
+        # Get P1 smartmeter data from MODULES
         entity = self.gw_entities[entity_id]
         # !! DON'T CHANGE below two if-lines, will break stuff !!
         if self.smile_type == "power":
@@ -294,14 +278,13 @@ class SmileLegacyHelper(SmileCommon):
             if appliance.find("type").text in ACTUATOR_CLASSES:
                 self._get_actuator_functionalities(appliance, entity, data)
 
-        # Adam & Anna: the Smile outdoor_temperature is present in DOMAIN_OBJECTS and LOCATIONS - under Home
-        # The outdoor_temperature present in APPLIANCES is a local sensor connected to the active device
+        # Anna: the Smile outdoor_temperature is present in the Home location
+        # For some Anna's LOCATIONS is empty, falling back to domain_objects!
         if self._is_thermostat and entity_id == self.gateway_id:
-            outdoor_temperature = self._object_value(
-                self._home_location, "outdoor_temperature"
-            )
-            if outdoor_temperature is not None:
-                data.update({"sensors": {"outdoor_temperature": outdoor_temperature}})
+            locator = f"./location[@id='{self._home_loc_id}']/logs/point_log[type='outdoor_temperature']/period/measurement"
+            if (found := self._domain_objects.find(locator)) is not None:
+                value = format_measure(found.text, NONE)
+                data.update({"sensors": {"outdoor_temperature": value}})
                 self._count += 1
 
         if "c_heating_state" in data:
@@ -323,10 +306,10 @@ class SmileLegacyHelper(SmileCommon):
         search = self._modules
         mod_logs = search.findall("./module/services")
         for loc.measurement, loc.attrs in P1_LEGACY_MEASUREMENTS.items():
-            loc.meas_list = loc.measurement.split("_")
+            loc.meas_list = loc.measurement.partition("_")[0::2]
             for loc.logs in mod_logs:
                 for loc.log_type in mod_list:
-                    self._collect_power_values(data, loc, t_string, legacy=True)
+                    collect_power_values(data, loc, t_string, legacy=True)
 
         self._count += len(data["sensors"])
         return data
@@ -359,7 +342,7 @@ class SmileLegacyHelper(SmileCommon):
                     appl_i_loc.text, ENERGY_WATT_HOUR
                 )
 
-        self._count_data_items(data)
+        self._count = count_data_items(self._count, data)
 
     def _get_actuator_functionalities(
         self, xml: etree, entity: GwEntityData, data: GwEntityData
@@ -396,20 +379,6 @@ class SmileLegacyHelper(SmileCommon):
                 act_item = cast(ActuatorType, item)
                 data[act_item] = temp_dict
 
-    def _object_value(self, obj_id: str, measurement: str) -> float | int | None:
-        """Helper-function for smile.py: _get_entity_data().
-
-        Obtain the value/state for the given object from a location in DOMAIN_OBJECTS
-        """
-        val: float | int | None = None
-        search = self._domain_objects
-        locator = f'./location[@id="{obj_id}"]/logs/point_log[type="{measurement}"]/period/measurement'
-        if (found := search.find(locator)) is not None:
-            val = format_measure(found.text, NONE)
-            return val
-
-        return val
-
     def _preset(self) -> str | None:
         """Helper-function for smile.py: _climate_data().
 
